;Quest'opera e' stata rilasciata sotto la licenza Creative Commons
;Attribuzione-Non commerciale-Condividi allo stesso modo 2.5 Italia.
;
;Per leggere una copia della licenza visita il sito web
;http://creativecommons.org/licenses/by-nc-sa/2.5/it/ o spedisci una
;lettera a Creative Commons, 171 Second Street, Suite 300,
;San Francisco, California, 94105, USA.
;
;
;This work is licensed under the Creative Commons
;Attribution-Noncommercial-Share Alike 2.5 Italy License.
;
;To view a copy of this license, visit
;http://creativecommons.org/licenses/by-nc-sa/2.5/it/ or send a letter
;to Creative Commons, 171 Second Street, Suite 300, San Francisco,
;California, 94105, USA.

; ________________________________________________
; |          Programma principale 877            |
; |______________________________________________|
; |   Src File                                   |
; |______________________________________________|
; |   Tipo      : ASM                            |
; |   Versione  : 1                              |
; |   Release   : 0                              |
; |   Autore    : Lain Walter                    |
; |   Data      : 31/03/09                       |
; |______________________________________________|


        LIST P=16F877A, B=4, C=80, R=hex, X=ON

;#define _DEBUG

#ifdef _DEBUG
    __CONFIG 0x3772
#else
    __CONFIG 0x3F72
#endif

#include "./P16F877A.inc"
#include "./macro_16F.inc"

;***************************************************************************
;                               VARIABILI
;***************************************************************************

;---------------------------------------------------------------------Bank 0
        BANK_0
#include "src/def_timer.inc"

#include "src/def_io.inc"

#include "src/def_display.inc"

#include "src/def_pwm.inc"

#include "src/def_keyb.inc"
 BYTE LAST                                ;ultimo byte bank 0

;---------------------------------------------------------------------Bank 1
        BANK_1

;---------------------------------------------------------------------Bank 2
        BANK_2

;---------------------------------------------------------------------Bank 3
        BANK_3


#define        c_REV1        CHAR_1DP
#define        c_REV2        CHAR_0

;***************************************************************************
;                                   RESET
;***************************************************************************

 ORG RESET_VECTOR
        GOTO SYS_STARTUP


;***************************************************************************
;                                INTERRUPTS
;***************************************************************************

 ORG INTERRUPT_VECTOR

        PUSH                            ;
        BCF PCLATH,4                    ;Seleziona pagina 0
        BCF PCLATH,3                    ;
Tmr0_Int                                ;Gestione timer principale
        BANKSEL INTCON                  ;
        BTFSS INTCON,TMR0IF             ;Controlla se TIMER0 e' in OVF
        GOTO Tmr1_Int                   ; NO
        BCF INTCON,TMR0IF               ;clear flag IRQ
        BSF INTCON,TMR0IE               ;riabilita interrupt
        BSF f_IRQ_TIMER0_ON             ;Setta flag Timer0 IRQ
Tmr1_Int                                ;Gestione timer principale
        BANKSEL PIR1                    ;
        BTFSS PIR1,TMR1IF               ;Controlla se TIMER1 e' in OVF
        GOTO End_Int                    ; NO
        BCF PIR1,TMR1IF                 ; SI, clear flag IRQ
        BSF f_IRQ_TIMER1_ON             ;Setta flag Timer IRQ
        BCF T1CON,TMR1ON                ;Disattiva Timer1
        ADDLF TMR1H,c_TMR1_TIME_H       ;Reimposta durata Timer1
        ADDLF TMR1L,c_TMR1_TIME_L       ;
        BSF T1CON,TMR1ON                ;Attiva Timer1
End_Int
        BCF STATUS,RP0                  ;torna in Bank 0
        BCF STATUS,RP1                  ;
        POP                             ;
        RETFIE


;***************************************************************************
;                             START-UP SISTEMA
;***************************************************************************
SYS_STARTUP:

        SYS_INI                         ;Inizializza il sistema
        BANKSEL INTCON                  ;Bank 1
        BCF INTCON,GIE                  ;Disabilita tutti gli IRQ
        BCF INTCON,PEIE                 ;Disabilita IRQ periferiche
        BCF PIE2,CMIE                   ;Disabilita IRQ dei comparatori
        BANKSEL STATUS                  ;Bank 0
        CALL Timer_Init                 ;Init timer
        CALL IO_Init                    ;Init I/O
        CALL Disp_Init                  ;Init Display
        CALL PWM_Init                   ;Init PWM
        CALL Keyb_Init                  ;Init Tastiera

;*******************************************************

;***************************************************************************
;                           PROGRAMMA PRINCIPALE
;***************************************************************************
PROGRAM:
        MOVLW CHAR_8DP                  ;
        MOVWF l_DISP1_DATA              ;
        MOVWF l_DISP1_DATA+1            ;
        MOVWF l_DISP1_DATA+2            ;
        MOVLF CHAR_R,l_DISP1_DATA+3     ;Carica la revisione del firmware
        MOVLF c_REV1,l_DISP2_DATA       ; sul display 2, e attiva tutti
        MOVLF c_REV2,l_DISP2_DATA+1     ; gli altri segmenti e led
; abilita IRQ
        BCF INTCON,TMR0IF               ;
        BCF PIR1,TMR1IF                 ;Clear flag Timer IRQ
        BANKSEL PIE2                    ;
        BSF PIE1,TMR1IE                 ;Abilita IRQ del Timer1
        BCF PIE2,CMIE                   ;Disabilita IRQ dei comparatori
        BANKSEL PIR1                    ;
        BSF INTCON,PEIE                 ;Abilita Peripheral Interrupts
        SEI                             ;Abilita tutti gli interrupts
        BCF c_OEP_PORT,c_OEP_PIN        ;Attiva OE periferiche
        MOVLF c_TMR4_DIV,b_TMR4_CNT     ;carica timer4

;*******************************************************
;ATTESA INIZIALE (visualizza revisione)
;*******************************************************
        MOVLF .255,IR0H                 ;carica contatore step
Start_D BTFSS f_IRQ_TIMER1_ON           ;Controlla se avvenuto IRQ Timer1
        GOTO Start_D                    ;No, prosegue
        BCF f_IRQ_TIMER1_ON             ;Si, clear flag timer1 IRQ
        CALL Disp_Write                 ;refresh display
        DECFSZ IR0H,f                   ;
        GOTO Start_D                    ;loop attesa iniziale

;CICLO PRINCIPALE. Il micro continua a ciclare qui' di seguito
Main_Clk
        BTFSC f_IRQ_TIMER0_ON           ;Controlla se avvenuto IRQ Timer0
        CALL PWM_ACSin                  ; (gestione AC): SI
        BTFSS f_IRQ_TIMER1_ON           ;Controlla se avvenuto IRQ Timer1
        GOTO Main_Clk                   ;No, cicla

;*******************************************************
;CICLO VELOCE
;*******************************************************
        BCF f_IRQ_TIMER1_ON             ;Si, clear flag timer1 IRQ
        CALL Disp_Write                 ;refresh display
        CALL Keyb_Read                  ;lettura tastiera

;*******************************************************
;CICLO LENTO
;*******************************************************
        DECFSZ b_TMR4_CNT,f             ;decrementa timer lento
        GOTO Main_Clk                   ; se non a zero, torna a inizio ciclo
        MOVLF c_TMR4_DIV,b_TMR4_CNT     ;zero, ricarica timer
        BTFSS f_D_SHIFT                 ;controlla se richiesto shift display
        GOTO Main_Clk                   ; no, torna a inizio ciclo
        MOVFF l_DISP_BUF5+2,R0H         ;carica lunghezza vettore
        MOVFF l_DISP_BUF5+3,R0L         ;recupera posizione shift
        CALL Disp1_Shift                ;
        MOVFF R0L,l_DISP_BUF5+3         ;salva posizione shift
        GOTO Main_Clk                   ;Torna a inizio ciclo


;***************************************************************************
;                               FILES INCLUDE
;***************************************************************************
;PAGINA 0
#include "src/asm_timer.inc"
#include "src/asm_io.inc"
#include "src/asm_display.inc"
#include "src/asm_pwm.inc"
#include "src/asm_keyb.inc"

        END
